<?php

declare(strict_types=1);

namespace SimpleSAML\Module\elixir;

use SimpleSAML\Configuration;
use SimpleSAML\Logger;
use SimpleSAML\Module\discopower\PowerIdPDisco;
use SimpleSAML\Module\elixir\discowarning\WarningConfiguration;

/**
 * This class implements a IdP discovery service.
 *
 * This module extends the DiscoPower IdP disco handler, so it needs to be avaliable and enabled and configured.
 *
 * It adds functionality of whitelisting and greylisting IdPs. for security reasons for blacklisting please manipulate
 * directly with metadata. In case of manual idps comment them out or in case of automated metadata fetching configure
 * blacklist in config-metarefresh.php
 */
class Disco extends PowerIdPDisco
{
    private const DEBUG_PREFIX = "elixir:Disco - :";

    public const CONFIG_FILE_NAME = 'module_elixir.php';

    public const AUTH_ID = 'AuthID';

    public const SAML_SP_SSO = 'saml:sp:sso';

    public const WARNING = 'warningAttributes';

    public const CONTINUE_URL = 'continueUrl';

    // ROOT CONFIGURATION ENTRY

    public const DISCO = 'disco';

    public const UPSTREAM_IDP_ENTITY_ID = 'upstream_idp_entity_id';

    // VARIABLES

    private $discoConfiguration;

    private $upstreamIdpEntityId;

    private $warningConfiguration = null;

    public function __construct(array $metadataSets, $instance)
    {
        parent::__construct($metadataSets, $instance);
        try {
            $elixirConfig = Configuration::getConfig(self::CONFIG_FILE_NAME);
            if (empty($elixirConfig->getValue(self::DISCO, []))) {
                Logger::error(self::DEBUG_PREFIX . "missing key '" . self::DISCO . "' in module config file");
                throw new \Exception();
            }
            $this->discoConfiguration = $elixirConfig->getConfigItem(self::DISCO);
        } catch (\Exception $ex) {
            Logger::error(self::DEBUG_PREFIX . "missing or invalid '" . self::CONFIG_FILE_NAME . "' config file");
            throw $ex;
        }

        $this->upstreamIdpEntityId = $this->discoConfiguration->getString(self::UPSTREAM_IDP_ENTITY_ID, '');
        if (empty($this->upstreamIdpEntityId)) {
            Logger::error(
                self::DEBUG_PREFIX
                . "missing setting of upstream IdP that should be used. Use option '"
                . self::UPSTREAM_IDP_ENTITY_ID
                . "' in the discovery page configuration block to set the EntityID of the IdP"
            );
        }
        if ($this->discoConfiguration->hasValue(WarningConfiguration::WARNING_CONFIG)) {
            $this->warningConfiguration = new WarningConfiguration();
        }
    }

    public function handleRequest()
    {
        $this->start();

        $continueUrl = self::buildContinueUrl(
            $this->spEntityId,
            $this->returnURL,
            $this->returnIdParam,
            $this->upstreamIdpEntityId
        );

        $t = new DiscoTemplate($this->config);
        $t->data[self::WARNING] = $this->warningConfiguration;
        $t->data[self::CONTINUE_URL] = $continueUrl;
        $t->show();
    }

    private static function buildContinueUrl(
        string $entityID,
        string $return,
        string $returnIDParam,
        string $idpEntityId
    ): string {
        return '?' .
            'entityID=' . urlencode($entityID) . '&' .
            'return=' . urlencode($return) . '&' .
            'returnIDParam=' . urlencode($returnIDParam) . '&' .
            'idpentityid=' . urlencode($idpEntityId);
    }
}
